﻿unit window.appletv;

interface

uses
  qtx.sysutils,
  qtx.classes,
  qtx.dom.types,
  qtx.dom.events,
  qtx.dom.graphics,
  qtx.dom.widgets,
  qtx.dom.theme,
  qtx.dom.application,
  qtx.dom.forms,
  qtx.dom.control.panel,
  qtx.dom.events.mouse;

type
  TfrmAppleTV = class(TQTXForm)
  protected
    procedure InitializeObject; override;
    procedure FinalizeObject; override;
    procedure StyleObject; override;

    procedure initATV(const AWidget: String);
  public

  end;

implementation uses qtx.dom.stylesheet;

procedure TfrmAppleTV.InitializeObject;
begin
  inherited;

  Style.background := 'linear-gradient(to bottom, #f6f7fc 0%, #d5e1e8 40%)';
  Style.height := "100%";
  Style.minHeight:= "100%";

  for var i := 0 to 6 do
    TQTXWidget.Create(self, procedure(AWidget: TQTXWidget)
    begin
      // You can have multiple layer in this widget. The images should be transparent in this case
      AWidget.Style.overflow := 'visible';
      AWidget.InnerHtml :=
       '<div class="QTXAtvWidget-layer" data-img="logo'+i.ToString+'.jpg"></div>';

      AWidget.CssClasses.ClassAdd('QTXAtvWidget');

      AWidget.AddMouseClickDelegate(procedure (Sender: TQTXDOMMouseDelegate; EventObj: JMouseEvent)
      begin
       Showmessage(AWidget.Name);
      end);

      TQTXDispatch.Execute( procedure ()
       begin
        InitATV(AWidget.Name);
       end, 10);
    end);
end;

procedure TfrmAppleTV.FinalizeObject;
begin
  inherited;
end;

procedure TfrmAppleTV.StyleObject;
begin
  inherited;
  PositionMode := TQTXWidgetPositionMode.cpInitial;
  DisplayMode := TQTXWidgetDisplayMode.cdBlock;
end;


procedure TfrmAppleTV.initATV(const AWidget: String);
begin
 asm
    var
      bd = document.getElementsByTagName('body')[0],
      supportsTouch = 'ontouchstart' in window || navigator.msMaxTouchPoints;

      var thisImg = document.getElementById(@AWidget),
        layerElems = thisImg.querySelectorAll('.QTXAtvWidget-layer'),
        totalLayerElems = layerElems.length;

      if (totalLayerElems <= 0) {
        return;
      }

      while (thisImg.firstChild) {
        thisImg.removeChild(thisImg.firstChild);
      }

      var containerHTML = document.createElement('div'),
        shineHTML = document.createElement('div'),
        shadowHTML = document.createElement('div'),
        layersHTML = document.createElement('div'),
        layers = [];

      thisImg.id = 'QTXAtvWidget__' + @AWidget;
      containerHTML.className = 'QTXAtvWidget-container';
      shineHTML.className = 'QTXAtvWidget-shine';
      shadowHTML.className = 'QTXAtvWidget-shadow';
      layersHTML.className = 'QTXAtvWidget-layers';

      for (var i = 0; i < totalLayerElems; i++) {
        var layer = document.createElement('div'),
          imgSrc = layerElems[i].getAttribute('data-img');

        layer.className = 'QTXAtvWidget-rendered-layer';
        layer.setAttribute('data-layer', i);
        layer.style.backgroundImage = 'url(' + imgSrc + ')';
        layersHTML.appendChild(layer);

        layers.push(layer);
      }

      containerHTML.appendChild(shadowHTML);
      containerHTML.appendChild(layersHTML);
      containerHTML.appendChild(shineHTML);
      thisImg.appendChild(containerHTML);

      var w = thisImg.clientWidth || thisImg.offsetWidth || thisImg.scrollWidth;
      thisImg.style.transform = 'perspective(' + w * 3 + 'px)';

      if (supportsTouch) {
        window.preventScroll = false;

        (function(_thisImg, _layers, _totalLayers, _shine) {
          thisImg.addEventListener('touchmove', function(e) {
            if (window.preventScroll) {
              e.preventDefault();
            }
            processMovement(e, true, _thisImg, _layers, _totalLayers, _shine);
          });
          thisImg.addEventListener('touchstart', function(e) {
            window.preventScroll = true;
            processEnter(e, _thisImg);
          });
          thisImg.addEventListener('touchend', function(e) {
            window.preventScroll = false;
            processExit(e, _thisImg, _layers, _totalLayers, _shine);
          });
        })(thisImg, layers, totalLayerElems, shineHTML);
      } else {
        (function(_thisImg, _layers, _totalLayers, _shine) {
          thisImg.addEventListener('mousemove', function(e) {
            processMovement(e, false, _thisImg, _layers, _totalLayers, _shine);
          });
          thisImg.addEventListener('mouseenter', function(e) {
            processEnter(e, _thisImg);
          });
          thisImg.addEventListener('mouseleave', function(e) {
            processExit(e, _thisImg, _layers, _totalLayers, _shine);
          });
        })(thisImg, layers, totalLayerElems, shineHTML);
      }

    function processMovement(e, touchEnabled, elem, layers, totalLayers, shine) {

      var bdst = bd.scrollTop,
        bdsl = bd.scrollLeft,
        pageX = (touchEnabled) ? e.touches[0].pageX : e.pageX,
        pageY = (touchEnabled) ? e.touches[0].pageY : e.pageY,
        offsets = elem.getBoundingClientRect(),
        w = elem.clientWidth || elem.offsetWidth || elem.scrollWidth, // width
        h = elem.clientHeight || elem.offsetHeight || elem.scrollHeight, // height
        wMultiple = 320 / w,
        offsetX = 0.52 - (pageX - offsets.left - bdsl) / w, //cursor position X
        offsetY = 0.52 - (pageY - offsets.top - bdst) / h, //cursor position Y
        dy = (pageY - offsets.top - bdst) - h / 2, //@h/2 = center of container
        dx = (pageX - offsets.left - bdsl) - w / 2, //@w/2 = center of container
        yRotate = (offsetX - dx) * (0.07 * wMultiple), //rotation for container Y
        xRotate = (dy - offsetY) * (0.1 * wMultiple), //rotation for container X
        imgCSS = 'rotateX(' + xRotate + 'deg) rotateY(' + yRotate + 'deg)', //img transform
        arad = Math.atan2(dy, dx), //angle between cursor and center of container in RAD
        angle = arad * 180 / Math.PI - 90; //convert rad in degrees

      //get angle between 0-360
      if (angle < 0) {
        angle = angle + 360;
      }

      //container transform
      if (elem.firstChild.className.indexOf(' over') != -1) {
        imgCSS += ' scale3d(1.07,1.07,1.07)';
      }
      elem.firstChild.style.transform = imgCSS;

      //gradient angle and opacity for shine
      shine.style.background = 'linear-gradient(' + angle + 'deg, rgba(255,255,255,' + (pageY - offsets.top - bdst) / h * 0.4 + ') 0%,rgba(255,255,255,0) 80%)';
      shine.style.transform = 'translateX(' + (offsetX * totalLayers) - 0.1 + 'px) translateY(' + (offsetY * totalLayers) - 0.1 + 'px)';

      //parallax for each layer
      var revNum = totalLayers;
      for (var ly = 0; ly < totalLayers; ly++) {
        layers[ly].style.transform = 'translateX(' + (offsetX * revNum) * ((ly * 2.5) / wMultiple) + 'px) translateY(' + (offsetY * totalLayers) * ((ly * 2.5) / wMultiple) + 'px)';
        revNum--;
      }
    }

    function processEnter(e, elem) {
      elem.firstChild.className += ' over';
    }

    function processExit(e, elem, layers, totalLayers, shine) {

      var container = elem.firstChild;

      container.className = container.className.replace(' over', '');
      container.style.transform = '';
      shine.style.cssText = '';

      for (var ly = 0; ly < totalLayers; ly++) {
        layers[ly].style.transform = '';
      }

    }
  end;
end;

initialization
begin
  var lSheet := TQTXCSSRules.Global;
  lSheet.AddStyles(
     #'.QTXAtvWidget {
        display: inline-block;
        width: 280px;
        height: 158px;
        margin: 1em;
        border-radius: 5px;
        transform-style: preserve-3d;
        -webkit-tap-highlight-color: rgba(0, 0, 0, 0);
        cursor: pointer;
        background-color: transparent;
        color: transparent;

      }
      .QTXAtvWidget.depressed {
        margin-top: 25px;
        box-shadow: 0 5px 30px rgba(0, 0, 0, 0.4);
      }
      .QTXAtvWidget-img {
        border-radius: 5px;
        box-shadow: 0 2px 8px rgba(14, 21, 47, 0.25);
      }

      .QTXAtvWidget-container {
        position: relative;
        width: 100%;
        height: 100%;
        border-radius: 5px;
        transition: all 0.2s ease-out;
      }

      .QTXAtvWidget-container.over .QTXAtvWidget-shadow {
        box-shadow: 0 45px 100px rgba(14, 21, 47, 0.4), 0 16px 40px rgba(14, 21, 47, 0.4);
      }

      .QTXAtvWidget-layers {
        position: relative;
        width: 100%;
        height: 100%;
        border-radius: 5px;
        overflow: hidden;
        transform-style: preserve-3d;
      }

      .QTXAtvWidget-rendered-layer {
        position: absolute;
        width: 104%;
        height: 104%;
        top: -2%;
        left: -2%;
        background-repeat: no-repeat;
        background-position: center;
        background-color: transparent;
        background-size: cover;
        transition: all 0.1s ease-out;
      }

      .QTXAtvWidget-shadow {
        position: absolute;
        top: 5%;
        left: 5%;
        width: 90%;
        height: 90%;
        transition: all 0.2s ease-out;
        box-shadow: 0 8px 30px rgba(14, 21, 47, 0.6);
      }

      .QTXAtvWidget-shine {
        position: absolute;
        top: 0;
        left: 0;
        right: 0;
        bottom: 0;
        border-radius: 5px;
        background: linear-gradient(135deg, rgba(255, 255, 255, 0.25) 0%, rgba(255, 255, 255, 0) 40%);
      }
     ');
end;

end.
